import  Toast from "../toast/index.js"
import { isObeject, isFunction } from "@/utils/types.js"
import Validator from "@/utils/validate.js"
/**
 * @desc 请求 所有的状态码都在这里处理
 * @param { Object } options 参考uni.request参数说明
 * @return {
	status: true / false
	msg
	data
 }
 */
function validateOps(options = {}) {
	const validator = new Validator()
	validator.add(Object.keys(options), [{strategy: 'minLength:1', errorMsg: '参数格式不正确'}])
	validator.add(options.url, [{strategy: 'notEmpty', errorMsg: '请求地址不能为空'}, {strategy: 'isHttp', errorMsg: '请求地址不正确'}])
	return validator.validate()
}

// 请求
export function request (options) {
    const valite_err = validateOps(options)
	if(valite_err) return Toast({title: valite_err})
	options.method = options.method ? options.method.toUpperCase() : "GET" // 默认get
	options.needMask && uni.showLoading({title: '加载中...', mask: true})
    return new Promise((resolve, reject) => {
        uni.request({
            ...options,
            header: {
				'content-type': 'application/x-www-form-urlencoded',
				'cookie': 'JSESSIONID=aaaH_PdwNLO-Oan2L7QNx; path=/'
			},
            success: res => {
                const statusCode = res.statusCode
                // console.log(statusCode)
                switch(statusCode) {
                    case 200:
                        options.successCb && typeof options.successCb === 'function' && options.successCb()
                        // 处理某些接口返回String类型的数据
                        const data = (res.data && typeof res.data === 'string') ? JSON.parse(res.data) : res.data
                        const status = data.code
						switch(status) {
							case 10:
								resolve({
								    status: true,
								    data: data.list,
								    msg: data.rep_msg
								})
								break;
							case -3: 
								return Toast({title: 'login', cb: () => {
									uni.navigateTo({
										url: '/pages/login/index.vue'
									})
								}})
								break;
							default:
								Toast({title: data.rep_msg})
								reject({
									status: false,
									code: status, // 正常不返回为了好找 后端调试 -1 => 业务错误（一般失败情况返回此项）;-2 => 系统异常; -3 => 请先登录; -4 => 请求失败; -10 => 签名失败
									data: data.list,
									msg: data.rep_msg
								})
								break;
						}
                        
                        break;
                    case 404:
                        return Toast({title: '请求路径不存在'})                        
                        break;
                    case 500:
                        return Toast({title: '服务器错误'})
                        break; // 这里的break可以不写，不会执行到。return直接退出函数了
                    default:
                        Toast({title: '网络请求失败'})
                        return
                        break;    
                }
            },
            fail: err => {
                if(err.errMsg.includes('request:fail ')) {
                    Toast({title: err.errMsg})
                }
                options.errCb && typeof options.errCb === 'function' && options.errCb();
                reject({
                    status: false,
                    code: 400,
                    data: err
                })
            },
            complete: () => {
                options.completeCb && typeof options.completeCb === 'function' && options.completeCb();
				options.needMask && uni.hideLoading()
            }
        })
    })
}

// 上传文件
// export function upload() {
// 	return new Promise((resolve, reject) => {
// 		uni.uploadFile({
			
// 		})
// 	})
// }
